Attribute VB_Name = "modGrafoInfo"
Option Explicit
Public Enum enumParamGrafo
    FSM_FIRMA
    FSM_NAME
    FSM_GLOBAL_INIT
    FSM_GLOBAL_PRE_ACCION
    FSM_CONFIG
    FSM_PARAMETER
    FSM_GLOBAL_POS_ACCION
    FSM_GLOBAL_END
    FSM_MOD_HEADER
    FSM_FUN_HEADER
    FSM_GLOBAL_DEFAULT
    FSM_MAX_LOOPS
    FSM_RETURN_TYPE
End Enum

'Extrae la informacin del general del FSM
Public Function DimeDatoFSM(posicion As enumParamGrafo, grafo As AddFlow) As String
    DimeDatoFSM = ExtraeCampo(grafo.Tag, posicion)
End Function

Public Sub Configura_FSM(nombre As String, parametros As String, returnTypeFSM As String, inicializacion As String, _
                         preAccion As String, sDefault As String, posAccion As String, _
                         endAccion As String, version As String, config As String, _
                         moduleHeader As String, functionHeader As String, _
                         sMaxLoops As String, grafo As AddFlow)
    grafo.Tag = version & SEPARADOR & _
                Trim$(nombre) & SEPARADOR & _
                inicializacion & SEPARADOR & _
                preAccion & SEPARADOR & _
                config & SEPARADOR & _
                parametros & SEPARADOR & _
                posAccion & SEPARADOR & _
                endAccion & SEPARADOR & _
                moduleHeader & SEPARADOR & _
                functionHeader & SEPARADOR & _
                sDefault & SEPARADOR & _
                sMaxLoops & SEPARADOR & _
                returnTypeFSM & SEPARADOR
End Sub

Public Sub DameGrafoInfo(grafo As AddFlow, Optional cargaTag As Boolean = True)
Dim nodo As afNode
Dim hasChanged As Boolean

    'Localiza el nodo info para leerlo y borrarlo adems de fijar los flag .LogicalOnly
    hasChanged = grafo.IsChanged
    grafo.LogicalOnly = False
    For Each nodo In grafo.Nodes
        nodo.Marked = False
        If (nodo.UserData And NODO_VALIDO) = 0 Then
            nodo.Logical = False
        End If
        If nodo.UserData And NODO_INFO_DIAGRAM Then
            'Recupera la informacin
            If cargaTag Then
                grafo.Tag = nodo.Tag
                grafo.Font.Name = nodo.Font.Name
                grafo.Font.size = nodo.Font.size
                grafo.Font.Bold = nodo.Font.Bold
                grafo.Font.Italic = nodo.Font.Italic
                grafo.Font.Underline = nodo.Font.Underline
                grafo.Font.Strikethrough = nodo.Font.Strikethrough
                
                grafo.FillColor = nodo.FillColor
                grafo.ForeColor = nodo.ForeColor
            End If
            nodo.Marked = True
        End If
    Next
    grafo.DeleteMarked
    grafo.LogicalOnly = True
    grafo.SetChangedFlag hasChanged
End Sub
Public Sub CargaTextoNodos(grafo As AddFlow)
Dim nodo As afNode
Dim trans(0 To 5) As String
Dim estad(0 To 9) As String
    
    'Recupera info desde el .Tag al .Text
    For Each nodo In grafo.Nodes
        If EsEstado(nodo) Then
            'Extrae la informacin del nodo de estado
            ExtraeTodosCampos estad(), nodo.Tag
            Configura_Nodo_Estado estad(ESTADO_NAME), _
                                  estad(ACTIONS_MOORE), _
                                  estad(ACTIONS_ENTRY), _
                                  estad(ACTIONS_EXIT), _
                                  estad(ACTIONS_ELSE), _
                                  estad(ESTADO_INDICE), _
                                  nodo.UserData And NODO_ESTADO, _
                                  estad(FUNCTION_IS_IN), _
                                  estad(FUNCTION_GO_TO), _
                                  nodo
        
        ElseIf EsTransicion(nodo) Then
            'Extrae la informacin del nodo de transicion
            ExtraeTodosCampos trans(), nodo.Tag
            Configura_Nodo_Transicion trans(PRIORITY), _
                                      trans(CONDITIONS), _
                                      trans(ACTIONS_MEALY), _
                                      trans(ENTRY_EXIT_SELF_TRANS), _
                                      trans(MUST_BE_LAST), _
                                      trans(CONTINUE_IMMEDIATELY), _
                                      nodo
        End If
    Next
End Sub

Public Sub TomaGrafoInfo(frm As frmDoc, Optional bCleanNodes As Boolean = True)
Dim nodo As afNode
Dim grafo As AddFlow
Dim hasChanged As Boolean
Dim sSignature As String
    With frm
    
        sSignature = DimeDatoFSM(FSM_FIRMA, .AddFlow1)
        If Len(sSignature) = 0 Then
            sSignature = FIRMA_VALIDA
        End If

        'Store all the info that normally is not contained into addflow
        Configura_FSM FillSpaces(FileSinRutaNiExtension(.sFullDocname)), _
                      DimeDatoFSM(FSM_PARAMETER, .AddFlow1), _
                      DimeDatoFSM(FSM_RETURN_TYPE, .AddFlow1), _
                      DimeDatoFSM(FSM_GLOBAL_INIT, .AddFlow1), _
                      DimeDatoFSM(FSM_GLOBAL_PRE_ACCION, .AddFlow1), _
                      DimeDatoFSM(FSM_GLOBAL_DEFAULT, .AddFlow1), _
                      DimeDatoFSM(FSM_GLOBAL_POS_ACCION, .AddFlow1), _
                      DimeDatoFSM(FSM_GLOBAL_END, .AddFlow1), _
                      sSignature, _
                      .formatCompilacion & ";" & _
                      IIf(.mnuAutosizeNodes.Checked, "AS_Y;", "AS_N;") & _
                      IIf(.mnuFixedPitchOnly.Checked, "FO_Y;", "FO_N;") & _
                      IIf(.mnuHideComments.Checked, "HC_Y;", "HC_N;") & _
                      IIf(.bGeneraFuncInit, "FI_Y;", "FI_N;") & _
                      IIf(.bGeneraFuncEnd, "FE_Y;", "FE_N;") & _
                      IIf(.bDebugInfo, "DI_Y;", "DI_N;") & _
                      IIf(.bAnsiComment, "AC_Y;", "AC_N;") & _
                      IIf(.bTextBoxesInCode, "PT_Y;", "PT_N;") & _
                      IIf(.bHistoryFsm, "HF_Y;", "HF_N;") & _
                      IIf(.bUseBitFields, "BF_Y;", "BF_N;") & _
                      IIf(.bUseMacros, "UM_Y;", "UM_N;"), _
                      DimeDatoFSM(FSM_MOD_HEADER, .AddFlow1), _
                      DimeDatoFSM(FSM_FUN_HEADER, .AddFlow1), _
                      DimeDatoFSM(FSM_MAX_LOOPS, .AddFlow1), _
                      .AddFlow1
        Set grafo = .AddFlow1
    End With

    hasChanged = grafo.IsChanged

    ' Create a node used to store Control informations
    ' The position and size of the node are not important
    Set nodo = grafo.Nodes.Add(0, 0, 100, 100)
    nodo.Hidden = True
    nodo.Logical = False
    nodo.UserData = NODO_INFO_DIAGRAM
    nodo.Tag = grafo.Tag
    
    nodo.FillColor = grafo.FillColor
    nodo.ForeColor = grafo.ForeColor
'    'Ya se crea por defecto
'    nodo.Font.Name = grafo.Font.Name
'    nodo.Font.Size = grafo.Font.Size
'    nodo.Font.Bold = grafo.Font.Bold
'    nodo.Font.Italic = grafo.Font.Italic
'    nodo.Font.Underline = grafo.Font.Underline
'    nodo.Font.Strikethrough = grafo.Font.Strikethrough
    
    grafo.SetChangedFlag hasChanged
    
    If bCleanNodes Then
        'Elimina textos ya que la info ya reside en el .Tag
        For Each nodo In grafo.Nodes
            If (nodo.UserData And NODO_VALIDO) Then
                nodo.Text = vbNullString
                nodo.key = vbNullString
            End If
        Next
    End If
End Sub

